"""
contains all the questions that need to be asked by the games
"""
from prompt_toolkit.validation import Validator, ValidationError
import regex


class IntValidator(Validator):
    """
    thing
    """

    def validate(self, document) -> None:
        """
        validates the input
        """

        not_ok = regex.search('[^0-9]', document.text)
        if not_ok:
            raise ValidationError(
                message='Please enter a int',
                cursor_position=len(document.text)
            )


class OneToNineValidator(Validator):
    """
    thing
    """

    def validate(self, document) -> None:
        """
        validates the input
        """

        ok = regex.match('^[0-9]{1}$', document.text)
        if not ok:
            raise ValidationError(
                message='Please enter a int from one to nine',
                cursor_position=len(document.text)
            )


class StringValidator(Validator):
    """
        thing
        """

    def validate(self, document) -> None:
        """
        validates the input
        """

        ok = regex.match('^[a-zA-Z]{1,11}$', document.text)
        if not ok:
            raise ValidationError(
                message='Please enter a int from one to nine',
                cursor_position=len(document.text)
            )


tic_questions = [
    # question 1
    {
        'type': 'confirm',
        'message': 'Do you want to see the tutorial?',
        'name': 'tutorial',
        'default': True
    },
    # question 2
    {
        'type': 'list',
        'name': 'difficulty',
        'message': 'What difficulty level do you want?',
        'choices': ['Easy', 'Medium', 'Hard', 'Custom'],
    },
    # question 3
    {
        'type': 'input',
        'name': 'freq_lose',
        'message': 'frequency of losing moves:',
        'validate': IntValidator
    },
    # question 4
    {
        'type': 'input',
        'name': 'freq_draw',
        'message': 'frequency of drawing moves:',
        'validate': IntValidator
    },
    # question 5
    {
        'type': 'input',
        'name': 'freq_win',
        'message': 'frequency of winning moves:',
        'validate': IntValidator
    },
    # question 6
    {
        'type': 'confirm',
        'name': 'confirm_choices',
        'message': 'Are you sure about these choices?',
    },
    # question 7
    {
        'type': 'confirm',
        'name': 'human_or_ai',
        'message': 'Do you want to play against the AI?',
    },
    # question 8
    {
        'type': 'input',
        'name': 'picked_slot',
        'message': 'Pick a slot:',
        'validate': OneToNineValidator
    },
]

hangman_questions = [
    {
        'type': 'input',
        'message': 'Type your guess:',
        'name': 'hangman_guess',
        'validate': StringValidator
    },
]

rps_questions = [
    {
        'type': 'list',
        'name': 'rps',
        'message': 'Pick rock paper or scissors: ',
        'choices': ['Rock', 'Paper', 'Scissors'],
    },
]



"""
this is the tic tac toe game
"""
import random

from PyInquirer import prompt
from examples import custom_style_1
from clint.textui import colored

from game_hub.utils.logging import MyLogger
from game_hub.questions import tic_questions

logger = MyLogger.logging.getLogger('gamehub.games.ticTacToe')

board_keys = [
    "top_left", "top_mid", "top_right",
    "mid_left", "mid_mid", "mid_right",
    "bot_left", "bot_mid", "bot_right"
]
switch_player = {
    "player 1": "player 2",
    "player 2": "player 1"
}
potential_scores = {
    "player 1": 1,
    "tie": 0,
    "player 2": -1
}
evaluation_to_message = {
    "player 1": "player 1 won",
    "player 2": "player 2 won",
    "ai": "ai won",
    "tie": "It is a draw",
    "human": "you won"
}


def evaluate_board(board: dict, ai_player: str = None) -> str:
    """
    takes in a board as a dictionary and a list and then uses them to check who is the winner, loser or if it is a tie
    or the game hasn't finished yet.
    :param ai_player: if the user is playing against the ai, which player is the ai
    :param board: dictionary of all slots on the game board
    :return: the winning player, a tie or an empty string if the game is not finished
    """
    winner = ""
    list_board = list(board.values())

    for symbol in ["x", "o"]:
        row_1 = list_board[0] == list_board[1] == list_board[2] == symbol
        row_2 = list_board[3] == list_board[4] == list_board[5] == symbol
        row_3 = list_board[6] == list_board[7] == list_board[8] == symbol

        col_1 = list_board[0] == list_board[3] == list_board[6] == symbol
        col_2 = list_board[1] == list_board[4] == list_board[7] == symbol
        col_3 = list_board[2] == list_board[5] == list_board[8] == symbol

        dia_1 = list_board[0] == list_board[4] == list_board[8] == symbol
        dia_2 = list_board[2] == list_board[4] == list_board[6] == symbol
        if row_1 or row_2 or row_3 or col_1 or col_2 or col_3 or dia_1 or dia_2:
            winner = {'x': 'player 1', 'o': 'player 2', }[symbol]
            break
    if winner == "":
        for value in board.values():
            if value == "-":
                return ""
        return "tie"
    if ai_player:
        human_player = switch_player[ai_player]
        if ai_player == winner:
            winner = "ai"
        if human_player == winner:
            winner = "human"
    return winner


def pick_ai_move(board: dict, current_player: str, ai_difficulty_lvl: str) -> int:
    """
    using the mini_max function, it loops through all the moves for this turn and then finds out the best one by calling
    mini_max, and dependant on who's turn it is, the computer either saves the best score by looking for the lowest
    score (meaning player two's best move) or the highest score (meaning player one's best move)
    :param ai_difficulty_lvl: The difficulty level of the ai
    :param board: the tic tac toe board
    :param current_player: player who has the turn
    """
    if current_player == "player 1":
        maximising_player = False
        symbol_to_put_in_slot = "x"
    else:
        maximising_player = True
        symbol_to_put_in_slot = "o"
    best_move = float("-inf")
    mini_best_score = float("-inf")
    max_best_score = float("inf")
    if all(x == "-" for x in board.values()):
        return random.randint(0, 8)
    while best_move == float("-inf"):
        for i in range(9):
            if board[board_keys[i]] == '-':
                board[board_keys[i]] = symbol_to_put_in_slot
                score = mini_max(board, maximising_player)
                if ai_difficulty_lvl == "win":
                    if score > mini_best_score and not maximising_player:
                        mini_best_score = score
                        best_move = i
                    if score < max_best_score and maximising_player:
                        max_best_score = score
                        best_move = i
                if ai_difficulty_lvl == "draw":
                    if score == 0 and not maximising_player:
                        mini_best_score = score
                        best_move = i
                    if score == 0 and maximising_player:
                        max_best_score = score
                        best_move = i
                if ai_difficulty_lvl == "lose":
                    if score == -1 and not maximising_player:
                        mini_best_score = score
                        best_move = i
                    if score == 1 and maximising_player:
                        mini_best_score = score
                        best_move = i
                board[board_keys[i]] = "-"
        if ai_difficulty_lvl == "draw":
            ai_difficulty_lvl = "win"
        if ai_difficulty_lvl == "lose":
            ai_difficulty_lvl = "draw"
    return best_move


def mini_max(board: dict, maximizing_player: bool) -> int:
    """
    Scoring system: 1 -> win | 0 -> draw | -1 -> lose

    Given a board and the maximising player, it loops through the board, checks for an empty space, and if there is
    then it puts x into it and then calls itself, but switches the maximising player, so it then puts an o into the
    space, this keeps on going till the board is completed and it does an evaluation of the score, this is then
    compared to the worst score for that player (X's would be -infinity, and vice versa) as the variable best_score
    and if the new evaluation is better than best_score, it is assigned to best_score. it goes on to the next
    possible ending and then compares that to the current score and if it's better replaces it. this keeps on going
    till all endings are done and the best ending is saved, then you go one up and then check the next branch in the
    tree. It then checks all these branches, gets the best branch and goes up again, and repeats until it get's back
    to the top.

    :param board: the game board
    :param maximizing_player: the current player
    :return: what will happen if the ai goes here
    """

    if evaluate_board(board) != "":
        return potential_scores[evaluate_board(board)]

    if maximizing_player:
        max_best_move = float('-inf')
        for i in range(9):
            if board[board_keys[i]] == '-':
                board[board_keys[i]] = "x"
                score = mini_max(board, False)
                board[board_keys[i]] = "-"
                if score > max_best_move:
                    max_best_move = score
        return max_best_move
    else:
        min_best_move = float('inf')
        for i in range(9):
            if board[board_keys[i]] == '-':
                board[board_keys[i]] = "o"
                score = mini_max(board, True)
                board[board_keys[i]] = "-"
                if score < min_best_move:
                    min_best_move = score
        return min_best_move


def print_board(board: dict):
    """
    saves the three rows of the tic, tac toe board and into separate variables and then prints them with a space between
    each board
    :return: nothing
    """
    i = 0
    for key, value in board.items():
        i += 1
        if value == "-":
            board[key] = i
        if value == "x":
            board[key] = (getattr(colored, 'red')('X'))
        if value == "o":
            board[key] = (getattr(colored, 'blue')('O'))

    return (f"""
    {board['top_left']}  {board['top_mid']}  {board['top_right']}
    {board['mid_left']}  {board['mid_mid']}  {board['mid_right']}
    {board['bot_left']}  {board['bot_mid']}  {board['bot_right']}
    """)


def tutorial() -> None:
    """
    Asks if they want to see the tutorial, and if they do prints a string explaining the game
    """
    user_wants_tutorial = prompt(tic_questions[0], style=custom_style_1).get("tutorial")
    if user_wants_tutorial:
        logger.info("""
    You pick the slot you want to pick using a number e.g

    -  -  -
    -  -  -
    -  -  -

    Is this:

    1  2  3
    4  5  6
    7  8  9

    To win get three in a row

    Here we'll show an example game so you don't screw up

    Player one it's your turn!

    Player one: 3

    -  -  x
    -  -  -
    -  -  -

    Player 2 it's your turn!

    Player 2: 6

    -  -  x
    -  -  o
    -  -  -

    ...And so on until one of the players get's three in a row

        """)


def choose_difficulty() -> list:
    """
    picks the ai difficulty or gives the option for the user to create their own difficulty
    :return: a list which decides the difficulty of the ai with three possible variables
    """
    user_difficulty_lvl = prompt(tic_questions[1], style=custom_style_1).get("difficulty")

    freq_lose = 2
    freq_draw = 6
    freq_win = 6
    if user_difficulty_lvl == "Hard":
        freq_lose = 1
        freq_draw = 0
        freq_win = 16
    elif user_difficulty_lvl == "Easy":
        freq_lose = 10
        freq_draw = 2
        freq_win = 2
    elif user_difficulty_lvl == "Custom":
        confirm_choices = False
        while not confirm_choices:
            freq_lose = int(prompt(tic_questions[2], style=custom_style_1).get("freq_lose"))
            freq_draw = int(prompt(tic_questions[3], style=custom_style_1).get("freq_draw"))
            freq_win = int(prompt(tic_questions[4], style=custom_style_1).get("freq_win"))
            confirm_choices = prompt(tic_questions[5], style=custom_style_1).get("confirm_choices")
    return list((["lose"] * freq_lose) + (["draw"] * freq_draw) + (["win"] * freq_win))


class TicTacToe:
    """
    holds the game code
    """

    def __init__(self) -> None:
        self.board = {
            "top_left": "-", "top_mid": "-", "top_right": "-",
            "mid_left": "-", "mid_mid": "-", "mid_right": "-",
            "bot_left": "-", "bot_mid": "-", "bot_right": "-"
        }
        self.top_row = f"{self.board['top_left']}  {self.board['top_mid']}  {self.board['top_right']}"
        self.mid_row = f"{self.board['mid_left']}  {self.board['mid_mid']}  {self.board['mid_right']}"
        self.bot_row = f"{self.board['bot_left']}  {self.board['bot_mid']}  {self.board['bot_right']}"
        self.end_state_of_game = ""
        self.list_picked_slots = []
        self.ai_difficulty_lvls = []

    def tictactoe_game(self) -> None:
        """
        checks if any one has one yet and if they have then return who is the winner, if no one has won then
        it picks the player who is going first, prints the board, asks the user to pick a slot and replaces the slot
        with the user's mark (x or o). After that, it checks if there is a tie (if so it breaks the loop) and if there
        isn't then it continues until the match has finished. Finally after the loop breaks it checks who won and then
        prints the winner.
        """
        list_of_players = ["player 1", "player 2"]
        tutorial()
        ai_player = ""
        user_has_picked_ai = prompt(tic_questions[6], style=custom_style_1).get("human_or_ai")
        current_player = random.choice(list_of_players)
        if user_has_picked_ai:
            ai_player = random.choice(list_of_players)
            self.ai_difficulty_lvls = choose_difficulty()

        while self.end_state_of_game == '':
            current_player = switch_player[current_player]
            print(print_board(dict(self.board)))
            if user_has_picked_ai and ai_player == current_player:
                logger.info("it is the AI's turn")
            elif not user_has_picked_ai:
                logger.info("it is " + current_player + "'s turn")
            else:
                logger.info("it is your turn")
            if user_has_picked_ai and current_player == ai_player:
                ai_difficulty_lvl = self.ai_difficulty_lvls[random.randint(0, len(self.ai_difficulty_lvls) - 1)]
                picked_slot = pick_ai_move(self.board, current_player, ai_difficulty_lvl) + 1
            else:
                picked_slot = int(prompt(tic_questions[7], style=custom_style_1).get("picked_slot"))
            self.list_picked_slots.append(picked_slot)
            player_symbol = {'player 1': 'x', 'player 2': 'o', }[current_player]
            self.board[board_keys[picked_slot - 1]] = player_symbol
            self.end_state_of_game = evaluate_board(self.board, ai_player=ai_player)

        print(print_board(dict(self.board)))
        logger.info(evaluation_to_message[self.end_state_of_game])


def game() -> None:
    """
    thing
    """
    logger.info("the tic tac toe game has been picked")
    tic_tac_toe = TicTacToe()
    tic_tac_toe.tictactoe_game()


if __name__ == '__main__':
    game()



"""
tests all functions in gameHub.py
"""
# import json

import pytest
# from pathlib import Path

import gameHub
from game_hub.games import hangMan, rockPaperScissors, ticTacToe

list_games = [rockPaperScissors.rps_game, hangMan.game, ticTacToe.game]


def test_generate() -> None:
    """
    tests generate() function by making two lists of randomly generated games. Then it tests if the results are games
    and not just any random value. If they aren't games, then it fails. Finally it tests if the randomly generated lists
    aren't the same. If they are the same, then it fails.
    """
    random_results_1 = []
    random_results_2 = []
    for _ in range(9):
        random_results_1.append(gameHub.generate())
        random_results_2.append(gameHub.generate())
    for random_result_1 in random_results_1:
        assert random_result_1 in list_games
    for random_result_2 in random_results_2:
        assert random_result_2 in list_games
    assert random_results_1 != random_results_2


@pytest.mark.parametrize("games,expected",
                         [(['tictactoe', 'rps'], ['tictactoe', 'rps']),
                          (['rps', 'hangman'], ['rps', 'hangman']),
                          (['tictactoe', 'hangman'], ['tictactoe', 'hangman']),
                          (['tictacto', 'hangman'], "tictacto is not an available game\n"
                                                    "Did you mean: tictactoe?")
                          ])
def test_play_random_game_from_list(games: list, expected: list) -> None:
    """
    tests play_random_game_from_list() function in gameHub.py
    """
    actual_output = gameHub.play_random_game_from_list(games)
    random_results_1 = []
    random_results_2 = []
    if not isinstance(expected, str):
        assert actual_output in expected
        for _ in range(20):
            random_results_1.append(gameHub.play_random_game_from_list(games))
            random_results_2.append(gameHub.play_random_game_from_list(games))
        assert random_results_1 != random_results_2
    else:
        assert actual_output == expected


def test_play_again() -> None:
    """
    tests the play_again function in gameHub.py
    :return:
    """
    pass
    # file = './test_data/mock_dir/mock_dir/test_lastGame.json'
    # Path.rmdir(Path('test_data/mock_dir'))
    # gameHub.play_again(Path(file))
    # with open(file, 'r') as json_file:
    #     json_file_content = json.load(json_file)



"""
Bonus:
- Make logs be pretty printed
"""
import os
import logging
import logging.config

from datetime import datetime
from pathlib import Path
from typing import Union


def configure_logger(name: str, log_path: Union[str, Path]) -> logging.Logger:
    """
    configures the logger
    """
    if isinstance(log_path, str):
        log_path = Path(log_path)
    log_path.parent.mkdir(parents=True, exist_ok=True)

    logging.config.dictConfig({
        'version': 1,
        'formatters': {
            'message': {'format': '%(message)s'},
            'default': {'format': '%(asctime)s - %(name)-27s:[%(lineno)-4d] - function:%(funcName)-20s - '
                                  '%(levelname)-8s - %(message)s',
                        'datefmt': '%Y/%m/%d %I:%M:%S %p'}
        },
        'handlers': {
            'print_to_console': {
                'level': 'INFO',
                'class': 'logging.StreamHandler',
                'formatter': 'message',
                'stream': 'ext://sys.stdout'
            },
            'log_to_file': {
                'level': 'DEBUG',
                'class': 'logging.handlers.RotatingFileHandler',
                'formatter': 'default',
                'filename': log_path,
                'backupCount': 0
            },
            'root': {
                'level': 'DEBUG',
                'class': 'logging.handlers.RotatingFileHandler',
                'formatter': 'default',
                'filename': os.path.splitext(str(log_path))[0] + '_root.log',
                'backupCount': 0
            }
        },
        'loggers': {
            name: {
                'level': 'DEBUG',
                'handlers': ['print_to_console', 'log_to_file'],
                'filemode': 'w'
            },
            '': {
                'level': 'DEBUG',
                'handlers': ['root'],
                'filemode': 'w'
            }
        }
    })
    return logging.getLogger(name)


logs_path = Path.home() / '.gameHub/system/logs/'
log_file = logs_path / f'gamehub_{datetime.today().date()}.log'
logger = configure_logger('gamehub', log_file)



"""
functions that handle bad inputs in specific circumstances
"""
import re
from game_hub.utils.logging import MyLogger


logger = MyLogger.logging.getLogger('gamehub.utils.inputHandlers')


def check_for_bad_input_strings(user_input: str) -> bool:
    """
    checks if the string is empty, and then if it isn't then checks if it's an int, and finally if it's not an int then
    it returns true
    :param user_input:
    :return:
    """
    if user_input:
        if re.search("[^0-9]", user_input):
            if re.search("[a-zA-Z]", user_input):
                return False
            else:
                logger.info("you put in a symbol, please try again")
        else:
            logger.info("you put in an int, please try again")
    else:
        logger.info("you put in nothing, please try again")
    return True


def check_for_bad_input_int(user_string: str, list_bad_inputs: list, cap: int or float) -> bool:
    """
    given a string, a list of exceptions and a cap, this function checks if the inputted string is an int, it is not
    in the list of exceptions and it is underneath a certain cap. The instructions to avoid the list of exceptions and
    the cap are in the descriptions of the respective parameters.
    :param cap: if the user wants to have a cap, then they can input an int, however if they don't then they can
    put in float("-inf") or float("inf") to stop them from being compared.
    :param user_string: the input that gets tested if it's bad
    :param list_bad_inputs: this list is to check that the input hasn't already been picked in the game
    :return: a boolean that is True if the the input is an int
    """
    if user_string:
        if re.search("[^a-zA-z]", user_string):
            if re.search("[0-9]", user_string):
                user_string = int(user_string)
                if user_string not in list_bad_inputs:
                    if user_string < cap:
                        return False
                    else:
                        logger.info("your input was higher than ten, please try again")
                else:
                    logger.info("your input has already been picked please try again")
            else:
                logger.info("your input had symbols, please try again")
        else:
            logger.info("Your input was a string, please try again")
    else:
        logger.info("you inputted nothing, please try again")
    return True



"""
Games cli
"""
import click
from pathlib import Path
import json

from game_hub.games import hangMan, ticTacToe, rockPaperScissors


def change_last_game(game: str) -> None:
    """
    changes the last game to the game just played
    :param game: the new last played game
    :return: nothing
    """
    last_game_path = Path.home() / '.gameHub/lastGame.json'
    try:
        json_file = json.loads(last_game_path.read_text())
        json_file['last_game'] = game
        with last_game_path.open(mode='w') as json_output:
            json.dump(json_file, json_output, indent=4)
    except FileNotFoundError:
        # creates file
        json_file = open(str(last_game_path), 'w')
        json.dump({'last_game': game}, json_file, indent=4)
        json_file.close()


@click.group('play', short_help='group of all games')
def play() -> None:
    """Contains all games"""
    pass


@play.command('hang', short_help='Plays Hangman game')
def hangman() -> None:
    """Plays Hangman game"""
    hangMan.game()
    change_last_game('hangman')


@play.command('rps', short_help='Plays Rock Paper Scissors game')
def rps() -> None:
    """Plays rock paper scissors game"""
    rockPaperScissors.game()
    change_last_game('rps')


@play.command('tic', short_help='Plays Tic Tac Toe game')
def tictactoe() -> None:
    """Plays Tic tac toe game"""
    ticTacToe.game()
    change_last_game('tictactoe')



"""
An interface to play multiple games
"""
import random
import json
from pathlib import Path
from difflib import get_close_matches
from typing import Callable

from game_hub.games import hangMan, ticTacToe, rockPaperScissors
from game_hub.games.gameCLI import change_last_game
from game_hub.utils.logging import MyLogger


logger = MyLogger.logging.getLogger('gamehub.gamehub')
gamehub_directory = Path.home() / '.gameHub'

last_game_played: Callable
last_game: Callable

game_interpreter = {
    "tictactoe": ticTacToe.game,
    "hangman": hangMan.game,
    "rps": rockPaperScissors.game
}
game_changer = {
    rockPaperScissors.rps_game: "rps",
    ticTacToe.game: "tictactoe",
    hangMan.game: "hangman"
}


def generate() -> Callable:
    """
    Pick Random Game to play
    :return: picked game
    """
    list_games = [rockPaperScissors.rps_game, hangMan.game, ticTacToe.game]
    picked_game = list_games[random.randint(0, 2)]
    change_last_game(game_changer[picked_game])
    return picked_game


def play_random_game_from_list(games: iter) -> Callable or str:
    """
    play's a random game from a given list. if one of the games is spelt wrong it prints the closest matching command
    in case they misspell them.
    :param games: list of games to randomly chose from
    """
    for game in games:
        if game not in game_interpreter.keys():
            closest_matching_command = get_close_matches(game, game_interpreter.keys())
            fail_message = f"{game} is not an available game"
            if closest_matching_command:
                fail_message += f"\nDid you mean: {closest_matching_command[0]}?"
            else:
                fail_message += "\nSee all games by entering gamehub play or just play if you are in a repl"
            return fail_message
    return random.choice(games)


def play_again(json_file_path: Path) -> None:
    """
    plays the last game you played
    """
    json_file_path.parent.mkdir(parents=True, exist_ok=True)
    logger.info("Playing the last game")
    try:
        config = json.loads(json_file_path.read_text())
        logger.info(config['last_game'])
        if not config["last_game"]:
            logger.info("you haven't played a game yet")
        else:
            game_interpreter[config["last_game"]]()
    except FileNotFoundError:
        logger.warning("you haven't played a game yet")
        with open(str(json_file_path), 'w') as lastGame:
            json.dump({"last_game": None}, lastGame, indent=4)



"""
Utilities related to file paths
"""
import os


def find_file(file_name: str, start_path: str = './') -> str:
    """
    Find a file recursively from the star_path and return if found
    :param file_name:
    :param start_path:
    :return:
    """
    for root, dirs, files in os.walk(start_path):
        if file_name in files:
            return os.path.join(root, file_name)



"""
this is the hangman game
"""
import random

from PyInquirer import prompt
from examples import custom_style_1

from game_hub.utils.file.path import find_file
from game_hub.utils.logging import MyLogger
from game_hub.questions import hangman_questions

logger = MyLogger.logging.getLogger('gamehub.games.hangMan')


def randomise(file_path=None) -> str:
    """
    returns a random word from the hangManWords.txt file
    :return: random word
    """
    if file_path is None:
        file_name = 'hangManWords.txt'
        file_path = find_file(file_name)
        if file_path is None:
            file_path = find_file(file_name, '../')
    with open(file_path, 'r') as hang_man_words:
        hang_man_words_string = hang_man_words.read()
    words = hang_man_words_string.split()
    random_word_picker = random.randint(0, len(words) - 1)
    return words[random_word_picker]


def hiding(word: str) -> str:
    """
    takes a word and replaces all characters with underscores
    :return: word with characters as underscores
    """
    hidden_word = ""
    counter = 0
    broken_up_random_word = list(word)
    number_of_letters_in_random_word: int = len(broken_up_random_word)
    while counter != number_of_letters_in_random_word:
        hidden_word += "_"
        counter += 1
    the_word = ' '.join(list(hidden_word))
    logger.info(the_word)
    return hidden_word


def hang_man_game(word: str, hidden_word: str) -> str:
    """
    it loops while the player hasn't won or lost yet, and when it loops it asked the user for a letter, an if it's
    in the word then it replaces the space where the letter was originally in the hidden word and then prints it, if
    they get it wrong then the function takes off one life and prints incorrect then the hidden word again. if the
    user types in the full word or they guess letter by letter they win, but lose if they run out of lives
    :param hidden_word: the word but all letters are replaced with underscores
    :param word: the word that has been picked
    """
    list_hidden_word = list(hidden_word)
    chances: int = 10
    listed_word = list(word)
    while word != hidden_word and chances > 0:
        guess = prompt(hangman_questions[0], style=custom_style_1).get("hangman_guess")
        if guess == word:
            break
        list_hidden_word, is_guess_correct = replace_space_with_guess(guess, listed_word, list_hidden_word)
        logger.info(" ".join(list_hidden_word))
        if not is_guess_correct:
            chances -= 1
            logger.info("incorrect!")
            if chances > 1:
                logger.info("You have " + str(chances) + " lives left")
            elif chances == 1:
                logger.info("You have " + str(chances) + " life left")
    if chances == 0:
        return "You failed,The word was %s" % word
    else:
        return "Congrats, you won!"


def replace_space_with_guess(guess: str, list_word: list, list_hidden_word: list) -> list and bool:
    """
    replaces underscore with guess if the guess is correct
    :return: None
    """
    was_the_guess_correct = False
    for i in range(len(list_word)):
        if guess == list_word[i]:
            was_the_guess_correct = True
            list_hidden_word[i] = guess
    return list_hidden_word, was_the_guess_correct


def game() -> None:
    """
    plays the hangman game
    :return:
    """
    logger.info("the hang man game has been picked")
    random_word = randomise()
    end_message = hang_man_game(random_word, hiding(random_word))
    if end_message[0] == "Y":
        logger.info(end_message.split(',')[0])
        logger.info(end_message.split(',')[1])
    else:
        logger.info(end_message)


if __name__ == '__main__':
    game()



"""
How to upload new version:
1. Delete dist
2. Up the version number
3. Create dist directory using command: python setup.py sdist bdist_wheel
4. Upload using command: twine upload dist/*
"""
import pathlib
import setuptools

from setuptools import setup

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

setup(
    name='GameHub',
    version='1.1.6',
    description='Hub of games to play from',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://gitlab.com/bGerma/game_hub',
    author='Brooklyn Germa',
    author_email='brooklyn.germa@gmail.com',
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
    install_requires=[
        'click-repl', 'click', 'tabulate', 'PyInquirer', 'clint', 'prompt-toolkit==1.0.14', 'pyfiglet'
    ],
    packages=setuptools.find_packages(exclude=["*.test", "*.test.*", "test.*", "test"]),
    py_modules=['gameHub', 'gameHubCLI'],
    entry_points='''
        [console_scripts]
        gameHub=gameHubCLI:cli
    ''',
    include_package_data=True
)



"""
this is the rock paper scissors game
"""
import random

from PyInquirer import prompt
from examples import custom_style_1

from game_hub.utils.logging import MyLogger
from game_hub.questions import rps_questions


logger = MyLogger.logging.getLogger('gamehub.games.hangMan')


def check_who_won(user_choice: str, computer_choice: str) -> str:
    """
    given a user input and computer guess it compares them to see who would have won in a rock paper scissors match,
    note that the user input has to be at minimum transformed into either 'S', 'R' or 'P' for 'Scissors', 'Rock', and
    'paper'
    :param computer_choice: the computer's choice
    :param user_choice: the user's choice
    :return:
    """

    if user_choice == "Rock":
        if computer_choice == "rock":
            return "we drew"
        elif computer_choice == "scissors":
            return "you win"
        else:
            return "you lose"

    if user_choice == "Scissors":
        if computer_choice == "rock":
            return "you lose"
        elif computer_choice == "scissors":
            return "we drew"
        else:
            return "you win"

    if user_choice == "Paper":
        if computer_choice == "rock":
            return "you win"
        elif computer_choice == "scissors":
            return "you lose"
        else:
            return "we drew"


def rps_game() -> str:
    """
    this method asks the user to user to input three strings, rock paper and scissors, and if they input something else
    then they ask again, after it has a good input the method picks one of them randomly and then compares against the
    user to see who won.
    :return: who won
    """
    computer_choice = ["rock", "paper", "scissors"][random.randint(0, 2)]

    logger.info("Type in one of the three choices and I'll print mine.")
    user_choice = prompt(rps_questions[0], style=custom_style_1).get("rps")
    logger.info("the computer picked: " + computer_choice)
    return check_who_won(user_choice, computer_choice)


def game() -> None:
    """
    Plays Rock Paper Scissors
    """
    logger.info("The rock paper scissors game has been picked")
    logger.info(rps_game())


if __name__ == '__main__':
    game()



"""
holds all commands
"""
import click
from click_repl import repl
from prompt_toolkit.history import FileHistory
from pathlib import Path

from pyfiglet import Figlet

from game_hub.utils.logging import MyLogger

import gameHub
from game_hub.games import gameCLI

logger = MyLogger.logging.getLogger('gamehub.gameHubCLI')

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group(context_settings=CONTEXT_SETTINGS)
def cli() -> None:
    """
    gamehub is an interface for the three games we currently have in stock, which are tictactoe, hangman and rock paper
    scissors. You can play these games by typing 'gamehub play [game]' you can see the names of the games by typing in
    'gamehub play'. If you are in the repl, the commands are the same but you don't include the gamehub. As well as this
    , in the repl, you can press the tab key and a list of options appears that you can pick from.
    """
    pass


@cli.command('generate', short_help="pick random game to play")
def click_generate() -> None:
    """
    Pick Random Game to play
    """
    gameHub.generate()()


@cli.command("last_game", short_help="plays the last game you played")
def play_again() -> None:
    """
    plays the last game you played
    """
    gameHub.play_again(Path.home() / '.gameHub/lastGame.json')


@cli.command("play_list", short_help="play's a random game from a list")
@click.option('--games', '-g', nargs=1, type=str, help="type -g or --games + a game and repeat to make list",
              multiple=True)
def play_from_list(games) -> None:
    """
    play's a random game from a list and prints the closest command if you misspell it.
    \f
    :param games: string will print
    """
    picked_game = gameHub.play_random_game_from_list(games)
    if picked_game not in gameHub.game_interpreter.keys():
        logger.info(picked_game)
    else:
        gameHub.game_interpreter[picked_game]()


@cli.command('repl', short_help="creates a repl and a exit command")
def create_repl() -> None:
    """
    creates a repl and a exit command
    """
    f = Figlet(font='larry3d')
    logger.info(f.renderText('gamehub'))

    @cli.command("exit", short_help="breaks you out of repl")
    def exit_repl() -> None:
        """
        breaks you out of repl
        """
        raise EOFError

    cli.add_command(exit_repl)
    command_history_file = gameHub.gamehub_directory / 'system/my_repl_history'
    command_history_file.parent.mkdir(parents=True, exist_ok=True)
    prompt_kwargs = {
        'history': FileHistory(command_history_file),
    }
    repl(click.get_current_context(), prompt_kwargs=prompt_kwargs)


cli.add_command(gameCLI.play)
